#ifndef jvmConvert
#define jvmConvert
 
/*
*****************************************
* File jvmConvert: handling conversions *
*****************************************
*/

/*
  Conversions in the Java Virtual Machine are handled by a specific set of bytecode instructions. As you 
  know, Java allows you to convert from one type of value to another either implicitly or explicitly. When a 
  conversion occurs, such as an integer to single-precision floating-point, one of these bytecodes is used. 
  Note:
  If the conversion is to a type of smaller bit width, truncation may 
  occur. There is no notification when truncation occurs.
*/


#include "jvmStack.c"
#include "jvmData.c"
  
  
/*PUB jb_i2l*/
/* Meaning: Integer-to-long integer conversion. */

/*PUB jb_i2f*/
/* Meaning: Integer to single float. */

/*PUB jb_i2d*/
/* Meaning: Integer to double float. */

/*PUB jb_l2i*/
/* Meaning: Long integer to integer. */

/*PUB jb_l2f*/
/* Meaning: Long integer to single float. */

/*PUB jb_l2d*/
/* Meaning: Long integer to double float. */

/*PUB jb_f2i*/
/* Meaning: Single float to integer. */

/*PUB jb_f2l*/
/* Meaning: Single float to long integer. */

/*PUB jb_f2d*/
/* Meaning: Single float to double float. */

/*PUB jb_d2i*/
/* Meaning: Double float to integer. */

/*PUB jb_d2l*/
/* Meaning: Double float to long integer. */

/*PUB jb_d2f*/
/* Meaning: Double float to single float. */


void i2b(void) {
  long w1;
/* Auxiliary function (no bytecode): Meaning: Integer to signed byte. */
  w1 = pop();
  w1 &= 0xFF;
  push(w1);
}  

void jb_i2b(void) {
  long w1;
/* Meaning: Integer to signed byte. */
/*
  JEM format: i2b
*/
  i2b();
  incJvmPC(1);
}  

void i2c(void) {
  long w1;
/* Auxiliary function (no bytecode): Meaning: Integer to char. */
  w1 = pop();
  w1 &= 0xFF;
  push(w1);
}

void jb_i2c(void) {
  long w1;
/* Meaning: Integer to char. */
/*
  JEM format: i2c
*/
  i2c();
  incJvmPC(1);
}


/*PUB jb_i2s*/
/* Meaning: Integer to short. */


long b2i(long bytevalue) {
/* Auxiliary function (no bytecode): Meaning: Signed byte to integer. */
  if (bytevalue & 0x80) return bytevalue | 0xFFFFFF00;
  else return bytevalue;
}  

long c2i(long charvalue) {
/* Auxiliary function (no bytecode): Meaning: Char to integer. */
  return charvalue & 0xFF;
}  

long z2i(long booleanvalue) {
/* Auxiliary function (no bytecode): Meaning: Boolean to integer. */
  if (booleanvalue == 0) return 0;
  else return -1;
}  

/*PUB jb_s2i*/
/* Auxiliary function (no bytecode): Meaning: Short to integer. */

#endif

