#ifndef jvmDefines
#define jvmDefines

/*
*****************************************
* File jvmDefines: constant definitions *
*****************************************
*/


//Virtual Peripheral types
#define  NONE   0
#define  TIMER  1
#define  ADC    2
#define  DAC    3
#define  PWM    4
#define  UARTRX 5
#define  UARTTX 6

#define  MAX_NUM_VPS  6
  
// The SX banks the VPs use.
#define  nmPeriph0Bank  0x80
#define  nmPeriph1Bank  (nmPeriph0Bank + 0x10)
#define  nmPeriph2Bank  (nmPeriph0Bank + 0x20)
#define  nmPeriph3Bank  (nmPeriph0Bank + 0x30)
#define  nmPeriph4Bank  (nmPeriph0Bank + 0x40)
#define  nmPeriph5Bank  (nmPeriph0Bank + 0x50)

//vector register (offsets within a VP bank) 
#define  nmPeriphISRVec  0x00  //each bank, holds VP type

//propeller fields for pinmasks in vpRambank slots
#define  nmInmask  16    //input pin 
#define  nmOutmask  20   //output pin
  
//file stamp\core\Timer.java (offsets within a VP bank)
#define  nmLatchTimer1   0x06  //32bit latch register
#define  nmLatchTimer2   0x07
#define  nmLatchTimer3   0x08
#define  nmLatchTimer4   0x09  
#define  nmTimer1        0x02  //32bit free running timer, incremented every 8.68 microsecond
#define  nmTimer2        0x03
#define  nmTimer3        0x04
#define  nmTimer4        0x05

//file stamp\core\ADC.java (offsets within a VP bank) 
#define  nmAdcValue  0x0B  // Current ADC value.
#define  nmInPort    0x0C  // 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
#define  nmInPin     0x0D  // bitmask (1 of 8 set)
#define  nmOutPort   0x0E  // 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
#define  nmOutPin    0x0F  // bitmask (1 of 8 set)
//unused registers used for accumulator, count and pin masks
#define  nmAdcAccu     0x0A
#define  nmAdcCount    0x01
//nmAdcInMask   0x02  //long-aligned input pin mask
//nmAdcOutMask  0x06  //long-aligned output pin mask
  
//file stamp\core\DAC.java (offsets within a VP bank) 
#define  nmDacValue  0x0B  // Current DAC value (dutycycle).
#define  nmDacPort   0x0E  // 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
#define  nmDacPin    0x0F  // bitmask (1 of 8 set)
//unused registers used for accumulator and output pin mask
#define  nmDacAccu     0x0A
//nmDacOutMask  0x06 //long-aligned output pin mask
  
//file stamp\core\PWM.java (offsets within a VP bank) 
#define  nmPWMPort      0x0E  // 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
#define  nmPWMPin       0x0F  // bitmask (1 of 8 set)
//unused registers used for count registers and output pin mask
#define  nmPWMhighTime  0x02
#define  nmPWMlowTime   0x03
#define  nmPWMhighZero  0x04
#define  nmPWMlowZero   0x05
//nmPWMoutMask   0x06 //long-aligned output pin mask 
#define  nmPWMCount     0x0A
#define  nmPWMState     0x0B
  
//file stamp\core\Uart.java (offsets within a VP bank) 
#define  nmUartDataH     0x02  // transmit register highbyte
#define  nmUartDataL     0x03  // transmit register lowbyte 
#define  nmUartInvert    0x05  // 0 for normal, 1 for invert
#define  nmUartBaud      0x06  // 1000000/(bps * 4.34)
#define  nmUartStopBits  0x07  // Tx only: 00000001 for 1 stop bit, 00000011 for 2 etc
#define  nmUartPort      0x0C  // 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
#define  nmUartPin       0x0D  // bit mask (1 of 8 set)
#define  nmUartHSPort    0x0E  // 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
#define  nmUartHSPin     0x0F  // bitmask (1 of 8 set)
#define  nmUartBufferH   0x0A  // object reference of 256 byte buffer (so buffer starts at offset +4) 
#define  nmUartBufferL   0x0B
#define  nmUartHead      0x08  //Index of the next free byte in the buffer. 
#define  nmUartTail      0x09  //Index of the oldest byte in the buffer.
                               //If tailPointer  headPointer the buffer is empty.
//unused registers for count and divide
#define  nmUartCount     0x01
#define  nmUartDivide    0x04
  
//Uart baud constants (divide values)
#define  speed600    192  //The speed value for 600 baud.
#define  speed1200   96   //The speed value for 1200 baud.
#define  speed2400   48   //The speed value for 2400 baud.
#define  speed4800   24   //The speed value for 4800 baud.
#define  speed7200   16   //The speed value for 7200 baud.
#define  speed9600   12   //The speed value for 9600 baud.
#define  speed14400  8    //The speed value for 14400 baud.
#define  speed19200  6    //The speed value for 19200 baud.
#define  speed28800  4    //The speed value for 28800 baud.
#define  speed38400  3    //The speed value for 38400 baud.
#define  speed57600  2    //The speed value for 57600 baud.

#endif
  