#ifndef jvmEngine
#define jvmEngine

/*
**********************************
* jvmEngine - executes bytecodes *
**********************************
*/


//first 64
#define  Jem_NOP               0x00
#define  Jem_ICONST_M1         0x01
#define  Jem_ICONST_0          0x02
#define  Jem_ICONST_1          0x03
#define  Jem_ICONST_2          0x04
#define  Jem_ICONST_3          0x05
#define  Jem_ICONST_4          0x06
#define  Jem_ICONST_5          0x07
#define  Jem_BIPUSH            0x08
#define  Jem_SIPUSH            0x09
#define  Jem_ILOAD             0x0A
#define  Jem_ILOAD_0           0x0B
#define  Jem_ILOAD_1           0x0C
#define  Jem_ILOAD_2           0x0D
#define  Jem_ILOAD_3           0x0E
#define  Jem_ISTORE            0x0F
#define  Jem_ISTORE_0          0x10
#define  Jem_ISTORE_1          0x11
#define  Jem_ISTORE_2          0x12
#define  Jem_ISTORE_3          0x13
#define  Jem_IFEQ              0x14
#define  Jem_IFNE              0x15
#define  Jem_IFLT              0x16
#define  Jem_IFLE              0x17
#define  Jem_IFGT              0x18
#define  Jem_IFGE              0x19
#define  Jem_IF_ICMPEQ         0x1A
#define  Jem_IF_ICMPNE         0x1B
#define  Jem_IF_ICMPLT         0x1C
#define  Jem_IF_ICMPGE         0x1D
#define  Jem_IF_ICMPGT         0x1E
#define  Jem_IF_ICMPLE         0x1F
#define  Jem_IF_ACMPEQ         0x20
#define  Jem_IF_ACMPNE         0x21
#define  Jem_IRETURN           0x22
#define  Jem_RETURN            0x23
#define  Jem_INVOKEVIRTUAL     0x24
#define  Jem_INVOKESTATIC      0x25
#define  Jem_GETSTATIC         0x26
#define  Jem_PUTSTATIC         0x27
#define  Jem_POP               0x28
#define  Jem_POP2              0x29
#define  Jem_DUP               0x2A
#define  Jem_DUP_X1            0x2B
#define  Jem_DUP_X2            0x2C
#define  Jem_DUP2              0x2D
#define  Jem_DUP2_X1           0x2E
#define  Jem_DUP2_X2           0x2F
#define  Jem_SWAP              0x30
#define  Jem_IADD              0x31
#define  Jem_ISUB              0x32
#define  Jem_INEG              0x33
#define  Jem_IOR               0x35
#define  Jem_ISHL              0x37
#define  Jem_ISHR              0x39
#define  Jem_IUSHR             0x3B
#define  Jem_IAND              0x3D
#define  Jem_IXOR              0x3F
#define  Jem_I2B               0x41
#define  Jem_GOTO              0x43
#define  Jem_INVOKEINTERFACE   0x45
#define  Jem_ARRAYLENGTH       0x47
#define  Jem_ATHROW            0x49
#define  Jem_CHECKCAST         0x4B
//next 64
#define  Jem_INSTANCEOF        0x4D
#define  Jem_WIDE              0x4F
#define  Jem_IINC              0x51
#define  Jem_GETFIELD          0x53
#define  Jem_PUTFIELD          0x55
#define  Jem_NEW               0x57
#define  Jem_NEWARRAY          0x59
#define  Jem_ANEWARRAY         0x5B
#define  Jem_IALOAD            0x5D
#define  Jem_IASTORE           0x5F
#define  Jem_BALOAD            0x61
#define  Jem_BASTORE           0x63
#define  Jem_LDC               0x65
#define  Jem_LDC_W             0x67
#define  Jem_LDC_W2            0x69
#define  Jem_IMUL              0x6B
#define  Jem_IDIV              0x6D
#define  Jem_IREM              0x6F
#define  Jem_JSR               0x71
#define  Jem_RET               0x73
#define  Jem_TABLESWITCH       0x75
#define  Jem_LOOKUPSWITCH      0x77

#include "jvmComm.c"
#include "jvmData.c"
#include "jvmStack.c"
#include "jvmConst.c"
#include "jvmLoad.c"
#include "jvmStore.c"
#include "jvmCompareJump.c"
#include "jvmInvoke.c"
#include "jvmMath.c"
#include "jvmArray.c"
#include "jvmObject.c"
#include "jvmSubReturn.c"
#include "jvmLogic.c"
#include "jvmConvert.c"
#include "jvmException.c"
  
   
long doStep(void) {
  long jemcode;
  //{{ perform a step }}
  jemcode = readJavaByte(getJvmPC());
  if (jemcode < Jem_INSTANCEOF) {
    switch (jemcode) {
      case Jem_NOP:             jb_nop();
	                            break;
      case Jem_ICONST_M1:       jb_iconst_m1();
	                            break;
      case Jem_ICONST_0:        jb_iconst_0();
	                            break;
      case Jem_ICONST_1:        jb_iconst_1();
	                            break;
      case Jem_ICONST_2:        jb_iconst_2();
	                            break;
      case Jem_ICONST_3:        jb_iconst_3();
	                            break;
      case Jem_ICONST_4:        jb_iconst_4();
	                            break;
      case Jem_ICONST_5:        jb_iconst_5();
	                            break;
      case Jem_BIPUSH:          jb_bipush();
	                            break;
      case Jem_SIPUSH:          jb_sipush();
	                            break;
      case Jem_ILOAD:           jb_iload();
	                            break;
      case Jem_ILOAD_0:         jb_iload_0();
	                            break;
      case Jem_ILOAD_1:         jb_iload_1();
	                            break;
      case Jem_ILOAD_2:         jb_iload_2();
	                            break;
      case Jem_ILOAD_3:         jb_iload_3();
	                            break;
      case Jem_ISTORE:          jb_istore();
	                            break;
      case Jem_ISTORE_0:        jb_istore_0();
	                            break;
      case Jem_ISTORE_1:        jb_istore_1();
	                            break;
      case Jem_ISTORE_2:        jb_istore_2();
	                            break;
      case Jem_ISTORE_3:        jb_istore_3();
	                            break;
      case Jem_IFEQ:            jb_ifeq();
	                            break;
      case Jem_IFNE:            jb_ifne();
	                            break;
      case Jem_IFLT:            jb_iflt();
	                            break;
      case Jem_IFLE:            jb_ifle();
	                            break;
      case Jem_IFGT:            jb_ifgt();
	                            break;
      case Jem_IFGE:            jb_ifge();
	                            break;
      case Jem_IF_ICMPEQ:       jb_if_icmpeq();
	                            break;
      case Jem_IF_ICMPNE:       jb_if_icmpne();
	                            break;
      case Jem_IF_ICMPLT:       jb_if_icmplt();
	                            break;
      case Jem_IF_ICMPGE:       jb_if_icmpge();
	                            break;
      case Jem_IF_ICMPGT:       jb_if_icmpgt();
	                            break;
      case Jem_IF_ICMPLE:       jb_if_icmple();
	                            break;
      case Jem_IF_ACMPEQ:       jb_if_acmpeq();
	                            break;
      case Jem_IF_ACMPNE:       jb_if_acmpne();
	                            break;
      case Jem_IRETURN:         jb_ireturn();
	                            break;
      case Jem_RETURN:          jb_return();
	                            break;
      case Jem_INVOKEVIRTUAL:   jb_invokevirtual();
	                            break;
      case Jem_INVOKESTATIC:    jb_invokestatic();
	                            break;
      case Jem_GETSTATIC:       jb_getstatic();
	                            break;
      case Jem_PUTSTATIC:       jb_putstatic();
	                            break;
      case Jem_POP:             jb_pop();
	                            break;
      case Jem_POP2:            jb_pop2();
	                            break;
      case Jem_DUP:             jb_dup();
	                            break;
      case Jem_DUP_X1:          jb_dup_x1();
	                            break;
      case Jem_DUP_X2:          jb_dup_x2();
	                            break;
      case Jem_DUP2:            jb_dup2();
	                            break;
      case Jem_DUP2_X1:         jb_dup2_x1();
	                            break;
      case Jem_DUP2_X2:         jb_dup2_x2();
	                            break;
      case Jem_SWAP:            jb_swap();
	                            break;
      case Jem_IADD:            jb_iadd();
	                            break;
      case Jem_ISUB:            jb_isub();
	                            break;
      case Jem_INEG:            jb_ineg();
	                            break;
      case Jem_IOR:             jb_ior();
	                            break;
      case Jem_ISHL:            jb_ishl();
	                            break;
      case Jem_ISHR:            jb_ishr();
	                            break;
      case Jem_IUSHR:           jb_iushr();
	                            break;
      case Jem_IAND:            jb_iand();
	                            break;
      case Jem_IXOR:            jb_ixor();
	                            break;
      case Jem_I2B:             jb_i2b();
	                            break;
      case Jem_GOTO:            jb_goto();
	                            break;
      case Jem_INVOKEINTERFACE: //not used by linker
	                            break;
      case Jem_ARRAYLENGTH:     jb_arraylength();
	                            break;
      case Jem_ATHROW:          jb_athrow();
	                            break;
      case Jem_CHECKCAST:       jb_checkcast();
	                            break;
      //default:
	}
  }
  else {
    switch (jemcode) {
      case Jem_INSTANCEOF:      jb_instanceof();
	                            break;
      case Jem_WIDE:            //not used by linker
	                            break;
      case Jem_IINC:            jb_iinc();
	                            break;
      case Jem_GETFIELD:        jb_getfield();
	                            break;
      case Jem_PUTFIELD:        jb_putfield();
	                            break;
      case Jem_NEW:             jb_new();
	                            break;
      case Jem_NEWARRAY:        jb_newarray();
	                            break;
      case Jem_ANEWARRAY:       jb_anewarray();
	                            break;
      case Jem_IALOAD:          jb_iaload();
	                            break;
      case Jem_IASTORE:         jb_iastore();
	                            break;
      case Jem_BALOAD:          jb_baload();
	                            break;
      case Jem_BASTORE:         jb_bastore();
	                            break;
      case Jem_LDC:             jb_ldc();
	                            break;
      case Jem_LDC_W:           jb_ldc_w();
	                            break;
      case Jem_LDC_W2:          //not used by linker
	                            break;
      case Jem_IMUL:            jb_imul();
	                            break;
      case Jem_IDIV:            jb_idiv();
	                            break;
      case Jem_IREM:            jb_irem();
	                            break;
      case Jem_JSR:             //not used by linker
	                            break;
      case Jem_RET:             //not used by linker
	                            break;
      case Jem_TABLESWITCH:     jb_tableswitch();
	                            break;
      case Jem_LOOKUPSWITCH:    jb_lookupswitch();
	                            break;
      //default:
	}
  }
  return -1; //no return in spin code
}

long doBytecode(void) {
  long state,c;
  /* Execute a bytecode */
  if ((getJvmPC() < 0x0014) && (getActivity() != kEnd)) { //0x0014 is address of first class header
    setActivity(kEnd);
    end();
  }
  state = getActivity();
  switch (state) {
    case kRun    : //JVM running
                   if (isBreakpoint()) {
                     breakpoint();
                     return -1;
                   }
				   else {
                     c = doStep();
                     if (mustNotifyStepDone())
                       sendStepDone();
                     return c;
				   }
				   break;
    case kBreak  : //appears not to be used
                   //so change state to kPassive
                   setActivity(kPassive);
                   return -1;
    case kPassive: //JVM was running but has stopped (exception,stopped,end)
                   if (mustStep()) {
                     c = doStep();
                     if (mustNotifyStepDone()) {
                       sendStepDone();
					 }
					 return c;
				   }
                   return -1;
    case kJog    : //appears not to be used
                   //so change state to kPassive
                   setActivity(kPassive);
                   return -1; 
    case kEnd    : //do nothing
                   return -1;
	//default:
  }  
  return -1;
}

          
#endif
      